from .base_classes import *
from .support_functions import *


class MessageSignificanceCategory(Element):
    """The impact of the content of a messageIf the element is present, it
    must have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(MessageSignificanceCategory, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if MessageSignificanceCategory.subclass:
            return MessageSignificanceCategory.subclass(*args_, **kwargs_)
        else:
            return MessageSignificanceCategory(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(MessageSignificanceCategory, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='MessageSignificanceCategory', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='MessageSignificanceCategory')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='MessageSignificanceCategory', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='MessageSignificanceCategory'):
        super(MessageSignificanceCategory, self).exportAttributes(outfile, level, already_processed, namespace_, name_='MessageSignificanceCategory')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='MessageSignificanceCategory', fromsubclass_=False, pretty_print=True):
        super(MessageSignificanceCategory, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='MessageSignificanceCategory', mapping_=None):
        element = super(MessageSignificanceCategory, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(MessageSignificanceCategory, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(MessageSignificanceCategory, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class MessageSignificanceCategory


class DocumentManifest(Resource):
    """A manifest that defines a set of documents.If the element is
    present, it must have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Resource
    def __init__(self, id=None, extension=None, modifierExtension=None, language=None, text=None, contained=None, masterIdentifier=None, identifier=None, subject=None, recipient=None, type_=None, author=None, created=None, source=None, status=None, supercedes=None, description=None, confidentiality=None, content=None):
        self.original_tagname_ = None
        super(DocumentManifest, self).__init__(id, extension, modifierExtension, language, text, contained, )
        self.masterIdentifier = masterIdentifier
        if identifier is None:
            self.identifier = []
        else:
            self.identifier = identifier
        if subject is None:
            self.subject = []
        else:
            self.subject = subject
        if recipient is None:
            self.recipient = []
        else:
            self.recipient = recipient
        self.type_ = type_
        if author is None:
            self.author = []
        else:
            self.author = author
        self.created = created
        self.source = source
        self.status = status
        self.supercedes = supercedes
        self.description = description
        self.confidentiality = confidentiality
        if content is None:
            self.content = []
        else:
            self.content = content
    def factory(*args_, **kwargs_):
        if DocumentManifest.subclass:
            return DocumentManifest.subclass(*args_, **kwargs_)
        else:
            return DocumentManifest(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_masterIdentifier(self): return self.masterIdentifier
    def set_masterIdentifier(self, masterIdentifier): self.masterIdentifier = masterIdentifier
    def get_identifier(self): return self.identifier
    def set_identifier(self, identifier): self.identifier = identifier
    def add_identifier(self, value): self.identifier.append(value)
    def insert_identifier(self, index, value): self.identifier[index] = value
    def get_subject(self): return self.subject
    def set_subject(self, subject): self.subject = subject
    def add_subject(self, value): self.subject.append(value)
    def insert_subject(self, index, value): self.subject[index] = value
    def get_recipient(self): return self.recipient
    def set_recipient(self, recipient): self.recipient = recipient
    def add_recipient(self, value): self.recipient.append(value)
    def insert_recipient(self, index, value): self.recipient[index] = value
    def get_type(self): return self.type_
    def set_type(self, type_): self.type_ = type_
    def get_author(self): return self.author
    def set_author(self, author): self.author = author
    def add_author(self, value): self.author.append(value)
    def insert_author(self, index, value): self.author[index] = value
    def get_created(self): return self.created
    def set_created(self, created): self.created = created
    def get_source(self): return self.source
    def set_source(self, source): self.source = source
    def get_status(self): return self.status
    def set_status(self, status): self.status = status
    def get_supercedes(self): return self.supercedes
    def set_supercedes(self, supercedes): self.supercedes = supercedes
    def get_description(self): return self.description
    def set_description(self, description): self.description = description
    def get_confidentiality(self): return self.confidentiality
    def set_confidentiality(self, confidentiality): self.confidentiality = confidentiality
    def get_content(self): return self.content
    def set_content(self, content): self.content = content
    def add_content(self, value): self.content.append(value)
    def insert_content(self, index, value): self.content[index] = value
    def hasContent_(self):
        if (
            self.masterIdentifier is not None or
            self.identifier or
            self.subject or
            self.recipient or
            self.type_ is not None or
            self.author or
            self.created is not None or
            self.source is not None or
            self.status is not None or
            self.supercedes is not None or
            self.description is not None or
            self.confidentiality is not None or
            self.content or
            super(DocumentManifest, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='DocumentManifest', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='DocumentManifest')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='DocumentManifest', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='DocumentManifest'):
        super(DocumentManifest, self).exportAttributes(outfile, level, already_processed, namespace_, name_='DocumentManifest')
    def exportChildren(self, outfile, level, namespace_='', name_='DocumentManifest', fromsubclass_=False, pretty_print=True):
        super(DocumentManifest, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.masterIdentifier is not None:
            self.masterIdentifier.export(outfile, level, namespace_, name_='masterIdentifier', pretty_print=pretty_print)
        for identifier_ in self.identifier:
            identifier_.export(outfile, level, namespace_, name_='identifier', pretty_print=pretty_print)
        for subject_ in self.subject:
            subject_.export(outfile, level, namespace_, name_='subject', pretty_print=pretty_print)
        for recipient_ in self.recipient:
            recipient_.export(outfile, level, namespace_, name_='recipient', pretty_print=pretty_print)
        if self.type_ is not None:
            self.type_.export(outfile, level, namespace_, name_='type', pretty_print=pretty_print)
        for author_ in self.author:
            author_.export(outfile, level, namespace_, name_='author', pretty_print=pretty_print)
        if self.created is not None:
            self.created.export(outfile, level, namespace_, name_='created', pretty_print=pretty_print)
        if self.source is not None:
            self.source.export(outfile, level, namespace_, name_='source', pretty_print=pretty_print)
        if self.status is not None:
            self.status.export(outfile, level, namespace_, name_='status', pretty_print=pretty_print)
        if self.supercedes is not None:
            self.supercedes.export(outfile, level, namespace_, name_='supercedes', pretty_print=pretty_print)
        if self.description is not None:
            self.description.export(outfile, level, namespace_, name_='description', pretty_print=pretty_print)
        if self.confidentiality is not None:
            self.confidentiality.export(outfile, level, namespace_, name_='confidentiality', pretty_print=pretty_print)
        for content_ in self.content:
            content_.export(outfile, level, namespace_, name_='content', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='DocumentManifest', mapping_=None):
        element = super(DocumentManifest, self).to_etree(parent_element, name_, mapping_)
        if self.masterIdentifier is not None:
            masterIdentifier_ = self.masterIdentifier
            masterIdentifier_.to_etree(element, name_='masterIdentifier', mapping_=mapping_)
        for identifier_ in self.identifier:
            identifier_.to_etree(element, name_='identifier', mapping_=mapping_)
        for subject_ in self.subject:
            subject_.to_etree(element, name_='subject', mapping_=mapping_)
        for recipient_ in self.recipient:
            recipient_.to_etree(element, name_='recipient', mapping_=mapping_)
        if self.type_ is not None:
            type__ = self.type_
            type__.to_etree(element, name_='type', mapping_=mapping_)
        for author_ in self.author:
            author_.to_etree(element, name_='author', mapping_=mapping_)
        if self.created is not None:
            created_ = self.created
            created_.to_etree(element, name_='created', mapping_=mapping_)
        if self.source is not None:
            source_ = self.source
            source_.to_etree(element, name_='source', mapping_=mapping_)
        if self.status is not None:
            status_ = self.status
            status_.to_etree(element, name_='status', mapping_=mapping_)
        if self.supercedes is not None:
            supercedes_ = self.supercedes
            supercedes_.to_etree(element, name_='supercedes', mapping_=mapping_)
        if self.description is not None:
            description_ = self.description
            description_.to_etree(element, name_='description', mapping_=mapping_)
        if self.confidentiality is not None:
            confidentiality_ = self.confidentiality
            confidentiality_.to_etree(element, name_='confidentiality', mapping_=mapping_)
        for content_ in self.content:
            content_.to_etree(element, name_='content', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(DocumentManifest, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'masterIdentifier':
            obj_ = Identifier.factory()
            obj_.build(child_)
            self.masterIdentifier = obj_
            obj_.original_tagname_ = 'masterIdentifier'
        elif nodeName_ == 'identifier':
            obj_ = Identifier.factory()
            obj_.build(child_)
            self.identifier.append(obj_)
            obj_.original_tagname_ = 'identifier'
        elif nodeName_ == 'subject':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.subject.append(obj_)
            obj_.original_tagname_ = 'subject'
        elif nodeName_ == 'recipient':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.recipient.append(obj_)
            obj_.original_tagname_ = 'recipient'
        elif nodeName_ == 'type':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.type_ = obj_
            obj_.original_tagname_ = 'type'
        elif nodeName_ == 'author':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.author.append(obj_)
            obj_.original_tagname_ = 'author'
        elif nodeName_ == 'created':
            obj_ = dateTime.factory()
            obj_.build(child_)
            self.created = obj_
            obj_.original_tagname_ = 'created'
        elif nodeName_ == 'source':
            obj_ = uri.factory()
            obj_.build(child_)
            self.source = obj_
            obj_.original_tagname_ = 'source'
        elif nodeName_ == 'status':
            obj_ = DocumentReferenceStatus.factory()
            obj_.build(child_)
            self.status = obj_
            obj_.original_tagname_ = 'status'
        elif nodeName_ == 'supercedes':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.supercedes = obj_
            obj_.original_tagname_ = 'supercedes'
        elif nodeName_ == 'description':
            obj_ = string.factory()
            obj_.build(child_)
            self.description = obj_
            obj_.original_tagname_ = 'description'
        elif nodeName_ == 'confidentiality':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.confidentiality = obj_
            obj_.original_tagname_ = 'confidentiality'
        elif nodeName_ == 'content':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.content.append(obj_)
            obj_.original_tagname_ = 'content'
        super(DocumentManifest, self).buildChildren(child_, node, nodeName_, True)
# end class DocumentManifest


class DocumentRelationshipType(Element):
    """The type of relationship between documentsIf the element is present,
    it must have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(DocumentRelationshipType, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if DocumentRelationshipType.subclass:
            return DocumentRelationshipType.subclass(*args_, **kwargs_)
        else:
            return DocumentRelationshipType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(DocumentRelationshipType, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='DocumentRelationshipType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='DocumentRelationshipType')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='DocumentRelationshipType', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='DocumentRelationshipType'):
        super(DocumentRelationshipType, self).exportAttributes(outfile, level, already_processed, namespace_, name_='DocumentRelationshipType')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='DocumentRelationshipType', fromsubclass_=False, pretty_print=True):
        super(DocumentRelationshipType, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='DocumentRelationshipType', mapping_=None):
        element = super(DocumentRelationshipType, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(DocumentRelationshipType, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(DocumentRelationshipType, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class DocumentRelationshipType


class InstanceAvailability(Element):
    """Availability of the resourceIf the element is present, it must have
    either a @value, an @id, or extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(InstanceAvailability, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if InstanceAvailability.subclass:
            return InstanceAvailability.subclass(*args_, **kwargs_)
        else:
            return InstanceAvailability(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(InstanceAvailability, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='InstanceAvailability', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='InstanceAvailability')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='InstanceAvailability', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='InstanceAvailability'):
        super(InstanceAvailability, self).exportAttributes(outfile, level, already_processed, namespace_, name_='InstanceAvailability')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='InstanceAvailability', fromsubclass_=False, pretty_print=True):
        super(InstanceAvailability, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='InstanceAvailability', mapping_=None):
        element = super(InstanceAvailability, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(InstanceAvailability, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(InstanceAvailability, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class InstanceAvailability


class IssueSeverity(Element):
    """How the issue affects the success of the actionIf the element is
    present, it must have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(IssueSeverity, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if IssueSeverity.subclass:
            return IssueSeverity.subclass(*args_, **kwargs_)
        else:
            return IssueSeverity(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(IssueSeverity, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='IssueSeverity', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='IssueSeverity')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='IssueSeverity', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='IssueSeverity'):
        super(IssueSeverity, self).exportAttributes(outfile, level, already_processed, namespace_, name_='IssueSeverity')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='IssueSeverity', fromsubclass_=False, pretty_print=True):
        super(IssueSeverity, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='IssueSeverity', mapping_=None):
        element = super(IssueSeverity, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(IssueSeverity, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(IssueSeverity, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class IssueSeverity


class BindingConformance(Element):
    """Binding conformance for applicationsIf the element is present, it
    must have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(BindingConformance, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if BindingConformance.subclass:
            return BindingConformance.subclass(*args_, **kwargs_)
        else:
            return BindingConformance(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(BindingConformance, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='BindingConformance', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='BindingConformance')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='BindingConformance', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='BindingConformance'):
        super(BindingConformance, self).exportAttributes(outfile, level, already_processed, namespace_, name_='BindingConformance')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='BindingConformance', fromsubclass_=False, pretty_print=True):
        super(BindingConformance, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='BindingConformance', mapping_=None):
        element = super(BindingConformance, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(BindingConformance, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(BindingConformance, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class BindingConformance


class ConstraintSeverity(Element):
    """SHALL applications comply with this constraint?If the element is
    present, it must have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(ConstraintSeverity, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if ConstraintSeverity.subclass:
            return ConstraintSeverity.subclass(*args_, **kwargs_)
        else:
            return ConstraintSeverity(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(ConstraintSeverity, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='ConstraintSeverity', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='ConstraintSeverity')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='ConstraintSeverity', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='ConstraintSeverity'):
        super(ConstraintSeverity, self).exportAttributes(outfile, level, already_processed, namespace_, name_='ConstraintSeverity')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='ConstraintSeverity', fromsubclass_=False, pretty_print=True):
        super(ConstraintSeverity, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='ConstraintSeverity', mapping_=None):
        element = super(ConstraintSeverity, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(ConstraintSeverity, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(ConstraintSeverity, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class ConstraintSeverity


class PropertyRepresentation(Element):
    """How a property is represented on the wireIf the element is present,
    it must have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(PropertyRepresentation, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if PropertyRepresentation.subclass:
            return PropertyRepresentation.subclass(*args_, **kwargs_)
        else:
            return PropertyRepresentation(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(PropertyRepresentation, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='PropertyRepresentation', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='PropertyRepresentation')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='PropertyRepresentation', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='PropertyRepresentation'):
        super(PropertyRepresentation, self).exportAttributes(outfile, level, already_processed, namespace_, name_='PropertyRepresentation')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='PropertyRepresentation', fromsubclass_=False, pretty_print=True):
        super(PropertyRepresentation, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='PropertyRepresentation', mapping_=None):
        element = super(PropertyRepresentation, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(PropertyRepresentation, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(PropertyRepresentation, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class PropertyRepresentation


class AggregationMode(Element):
    """How resource references can be aggregatedIf the element is present,
    it must have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(AggregationMode, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if AggregationMode.subclass:
            return AggregationMode.subclass(*args_, **kwargs_)
        else:
            return AggregationMode(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(AggregationMode, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='AggregationMode', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='AggregationMode')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='AggregationMode', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='AggregationMode'):
        super(AggregationMode, self).exportAttributes(outfile, level, already_processed, namespace_, name_='AggregationMode')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='AggregationMode', fromsubclass_=False, pretty_print=True):
        super(AggregationMode, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='AggregationMode', mapping_=None):
        element = super(AggregationMode, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(AggregationMode, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(AggregationMode, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class AggregationMode


class ExtensionContext(Element):
    """How an extension context is interpretedIf the element is present, it
    must have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(ExtensionContext, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if ExtensionContext.subclass:
            return ExtensionContext.subclass(*args_, **kwargs_)
        else:
            return ExtensionContext(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(ExtensionContext, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='ExtensionContext', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='ExtensionContext')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='ExtensionContext', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='ExtensionContext'):
        super(ExtensionContext, self).exportAttributes(outfile, level, already_processed, namespace_, name_='ExtensionContext')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='ExtensionContext', fromsubclass_=False, pretty_print=True):
        super(ExtensionContext, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='ExtensionContext', mapping_=None):
        element = super(ExtensionContext, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(ExtensionContext, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(ExtensionContext, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class ExtensionContext


class SlicingRules(Element):
    """How slices are interpreted when evaluating an instanceIf the element
    is present, it must have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(SlicingRules, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if SlicingRules.subclass:
            return SlicingRules.subclass(*args_, **kwargs_)
        else:
            return SlicingRules(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(SlicingRules, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='SlicingRules', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='SlicingRules')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='SlicingRules', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='SlicingRules'):
        super(SlicingRules, self).exportAttributes(outfile, level, already_processed, namespace_, name_='SlicingRules')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='SlicingRules', fromsubclass_=False, pretty_print=True):
        super(SlicingRules, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='SlicingRules', mapping_=None):
        element = super(SlicingRules, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(SlicingRules, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(SlicingRules, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class SlicingRules


class HierarchicalRelationshipType(Element):
    """Type indicating if this is a parent or child relationshipIf the
    element is present, it must have either a @value, an @id, or
    extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(HierarchicalRelationshipType, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if HierarchicalRelationshipType.subclass:
            return HierarchicalRelationshipType.subclass(*args_, **kwargs_)
        else:
            return HierarchicalRelationshipType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(HierarchicalRelationshipType, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='HierarchicalRelationshipType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='HierarchicalRelationshipType')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='HierarchicalRelationshipType', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='HierarchicalRelationshipType'):
        super(HierarchicalRelationshipType, self).exportAttributes(outfile, level, already_processed, namespace_, name_='HierarchicalRelationshipType')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='HierarchicalRelationshipType', fromsubclass_=False, pretty_print=True):
        super(HierarchicalRelationshipType, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='HierarchicalRelationshipType', mapping_=None):
        element = super(HierarchicalRelationshipType, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(HierarchicalRelationshipType, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(HierarchicalRelationshipType, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class HierarchicalRelationshipType


class FilterOperator(Element):
    """The kind of operation to perform as a part of a property based
    filterIf the element is present, it must have either a @value,
    an @id, or extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(FilterOperator, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if FilterOperator.subclass:
            return FilterOperator.subclass(*args_, **kwargs_)
        else:
            return FilterOperator(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(FilterOperator, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='FilterOperator', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='FilterOperator')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='FilterOperator', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='FilterOperator'):
        super(FilterOperator, self).exportAttributes(outfile, level, already_processed, namespace_, name_='FilterOperator')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='FilterOperator', fromsubclass_=False, pretty_print=True):
        super(FilterOperator, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='FilterOperator', mapping_=None):
        element = super(FilterOperator, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(FilterOperator, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(FilterOperator, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class FilterOperator



#Copyright (c) 1999 Christian Scholz (ruebe@aachen.heimat.de)
#
#This library is free software; you can redistribute it and/or
#modify it under the terms of the GNU Library General Public
#License as published by the Free Software Foundation; either
#version 2 of the License, or (at your option) any later version.
#
#This library is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#Library General Public License for more details.
#
#You should have received a copy of the GNU Library General Public
#License along with this library; if not, write to the Free
#Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
#MA 02111-1307, USA

"""
Python WebDAV Server.

This is an example implementation of a DAVserver using the DAV package.

"""


import sys
import logging

from pywebdav.lib.WebDAVServer import DAVRequestHandler
from pywebdav.lib.dbconn import Mconn

from .fshandler import FilesystemHandler

log = logging.getLogger()

class DAVAuthHandler(DAVRequestHandler):
    """
    Provides authentication based on parameters. The calling
    class has to inject password and username into this.
    (Variables: auth_user and auth_pass)
    """

    # Do not forget to set IFACE_CLASS by caller
    # ex.: IFACE_CLASS = FilesystemHandler('/tmp', 'http://localhost/')
    verbose = False

    def _log(self, message):
        if self.verbose:
            log.info(message)

    def get_userinfo(self,user,pw,command):
        """ authenticate user """

        if user == self._config.DAV.user and pw == self._config.DAV.password:
            log.info('Successfully authenticated user %s' % user)
            return 1

        log.info('Authentication failed for user %s' % user)
        return 0




# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    Copyright (C) 2008-2017 Luis Falcon <lfalcon@gnusolidario.org>
#    Copyright (C) 2011-2017 GNU Solidario <health@gnusolidario.org>
#
#
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from trytond.model import ModelView
from trytond.wizard import Wizard, StateTransition, StateAction, StateView, Button
from trytond.transaction import Transaction
from trytond.pool import Pool
from trytond.pyson import PYSONEncoder


__all__ = ['CreateInpatientEvaluation']

class CreateInpatientEvaluation(Wizard):
    'Create Inpatient Evaluation'
    __name__ = 'wizard.gnuhealth.inpatient.evaluation'
  
    start_state = 'inpatient_evaluation'
    inpatient_evaluation = StateAction('health_inpatient.act_inpatient_evaluation')

    def do_inpatient_evaluation(self, action):
      
        inpatient_registration = Transaction().context.get('active_id')

        try:
            reg_id = \
                Pool().get('gnuhealth.inpatient.registration').browse([inpatient_registration])[0]
        except:
            self.raise_user_error('no_record_selected')
            
        patient = reg_id.patient.id

        
        action['pyson_domain'] = PYSONEncoder().encode([
            ('patient', '=', patient),
            ('inpatient_registration_code', '=', reg_id.id),
            ('evaluation_type', '=', 'inpatient'),
            ])
        action['pyson_context'] = PYSONEncoder().encode({
            'patient': patient,
            'inpatient_registration_code': reg_id.id,
            'evaluation_type': 'inpatient',
            })
            
        return action, {}
        
    @classmethod
    def __setup__(cls):
        super(CreateInpatientEvaluation, cls).__setup__()
        cls._error_messages.update({
            'no_record_selected':
                'You need to select an inpatient registration record',
        })




# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    Copyright (C) 2008-2017 Luis Falcon <lfalcon@gnusolidario.org>
#    Copyright (C) 2011-2017 GNU Solidario <health@gnusolidario.org>
#
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from trytond.pool import Pool
from .health import *
from .wizard import *
from .report import *

def register():
    Pool.register(
        OperationalArea,
        OperationalSector,
        DomiciliaryUnit,
        Occupation,
        Ethnicity,
        BirthCertificate,
        DeathCertificate,
        Party,
        ContactMechanism,
        PersonName,
        PartyAddress,
        DrugDoseUnits,
        MedicationFrequency,
        DrugForm,
        DrugRoute,
        MedicalSpecialty,
        HealthInstitution,
        HealthInstitutionSpecialties,
        HealthInstitutionOperationalSector,
        HealthInstitutionO2M,
        HospitalBuilding,
        HospitalUnit,
        HospitalOR,
        HospitalWard,
        HospitalBed,
        HealthProfessional,
        HealthProfessionalSpecialties,        
        PhysicianSP,
        Family,
        FamilyMember,
        MedicamentCategory,
        Medicament,
        ImmunizationSchedule,
        ImmunizationScheduleLine,
        ImmunizationScheduleDose,
        PathologyCategory,
        PathologyGroup,
        Pathology,
        DiseaseMembers,
        BirthCertExtraInfo,
        DeathCertExtraInfo,
        DeathUnderlyingCondition,
        ProcedureCode,
        InsurancePlan,
        Insurance,
        AlternativePersonID,
        Product,
        GnuHealthSequences,
        PatientData,
        PatientDiseaseInfo,
        Appointment,
        AppointmentReport,
        OpenAppointmentReportStart,
        PatientPrescriptionOrder,
        PrescriptionLine,
        PatientMedication,
        PatientVaccination,
        PatientEvaluation,
        Directions,
        SecondaryCondition,
        DiagnosticHypothesis,
        SignsAndSymptoms,
        PatientECG,
        CheckImmunizationStatusInit,
        ProductTemplate,
        module='health', type_='model')
    Pool.register(
        OpenAppointmentReport,
        CreateAppointmentEvaluation,
        CheckImmunizationStatus,
        module='health', type_='wizard')
    Pool.register(
        PatientDiseaseReport,
        PatientMedicationReport,
        PatientVaccinationReport,
        ImmunizationStatusReport,
        module='health', type_='report')



from .base_classes import *
from .support_functions import *


class ImagingStudy(Resource):
    """Manifest of a set of images produced in study. The set of images may
    include every image in the study, or it may be an incomplete
    sample, such as a list of key images.If the element is present,
    it must have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Resource
    def __init__(self, id=None, extension=None, modifierExtension=None, language=None, text=None, contained=None, dateTime=None, subject=None, uid=None, accessionNo=None, identifier=None, order=None, modality=None, referrer=None, availability=None, url=None, numberOfSeries=None, numberOfInstances=None, clinicalInformation=None, procedure=None, interpreter=None, description=None, series=None):
        self.original_tagname_ = None
        super(ImagingStudy, self).__init__(id, extension, modifierExtension, language, text, contained, )
        self.dateTime = dateTime
        self.subject = subject
        self.uid = uid
        self.accessionNo = accessionNo
        if identifier is None:
            self.identifier = []
        else:
            self.identifier = identifier
        if order is None:
            self.order = []
        else:
            self.order = order
        if modality is None:
            self.modality = []
        else:
            self.modality = modality
        self.referrer = referrer
        self.availability = availability
        self.url = url
        self.numberOfSeries = numberOfSeries
        self.numberOfInstances = numberOfInstances
        self.clinicalInformation = clinicalInformation
        if procedure is None:
            self.procedure = []
        else:
            self.procedure = procedure
        self.interpreter = interpreter
        self.description = description
        if series is None:
            self.series = []
        else:
            self.series = series
    def factory(*args_, **kwargs_):
        if ImagingStudy.subclass:
            return ImagingStudy.subclass(*args_, **kwargs_)
        else:
            return ImagingStudy(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_dateTime(self): return self.dateTime
    def set_dateTime(self, dateTime): self.dateTime = dateTime
    def get_subject(self): return self.subject
    def set_subject(self, subject): self.subject = subject
    def get_uid(self): return self.uid
    def set_uid(self, uid): self.uid = uid
    def get_accessionNo(self): return self.accessionNo
    def set_accessionNo(self, accessionNo): self.accessionNo = accessionNo
    def get_identifier(self): return self.identifier
    def set_identifier(self, identifier): self.identifier = identifier
    def add_identifier(self, value): self.identifier.append(value)
    def insert_identifier(self, index, value): self.identifier[index] = value
    def get_order(self): return self.order
    def set_order(self, order): self.order = order
    def add_order(self, value): self.order.append(value)
    def insert_order(self, index, value): self.order[index] = value
    def get_modality(self): return self.modality
    def set_modality(self, modality): self.modality = modality
    def add_modality(self, value): self.modality.append(value)
    def insert_modality(self, index, value): self.modality[index] = value
    def get_referrer(self): return self.referrer
    def set_referrer(self, referrer): self.referrer = referrer
    def get_availability(self): return self.availability
    def set_availability(self, availability): self.availability = availability
    def get_url(self): return self.url
    def set_url(self, url): self.url = url
    def get_numberOfSeries(self): return self.numberOfSeries
    def set_numberOfSeries(self, numberOfSeries): self.numberOfSeries = numberOfSeries
    def get_numberOfInstances(self): return self.numberOfInstances
    def set_numberOfInstances(self, numberOfInstances): self.numberOfInstances = numberOfInstances
    def get_clinicalInformation(self): return self.clinicalInformation
    def set_clinicalInformation(self, clinicalInformation): self.clinicalInformation = clinicalInformation
    def get_procedure(self): return self.procedure
    def set_procedure(self, procedure): self.procedure = procedure
    def add_procedure(self, value): self.procedure.append(value)
    def insert_procedure(self, index, value): self.procedure[index] = value
    def get_interpreter(self): return self.interpreter
    def set_interpreter(self, interpreter): self.interpreter = interpreter
    def get_description(self): return self.description
    def set_description(self, description): self.description = description
    def get_series(self): return self.series
    def set_series(self, series): self.series = series
    def add_series(self, value): self.series.append(value)
    def insert_series(self, index, value): self.series[index] = value
    def hasContent_(self):
        if (
            self.dateTime is not None or
            self.subject is not None or
            self.uid is not None or
            self.accessionNo is not None or
            self.identifier or
            self.order or
            self.modality or
            self.referrer is not None or
            self.availability is not None or
            self.url is not None or
            self.numberOfSeries is not None or
            self.numberOfInstances is not None or
            self.clinicalInformation is not None or
            self.procedure or
            self.interpreter is not None or
            self.description is not None or
            self.series or
            super(ImagingStudy, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='ImagingStudy', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='ImagingStudy')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='ImagingStudy', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='ImagingStudy'):
        super(ImagingStudy, self).exportAttributes(outfile, level, already_processed, namespace_, name_='ImagingStudy')
    def exportChildren(self, outfile, level, namespace_='', name_='ImagingStudy', fromsubclass_=False, pretty_print=True):
        super(ImagingStudy, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.dateTime is not None:
            self.dateTime.export(outfile, level, namespace_, name_='dateTime', pretty_print=pretty_print)
        if self.subject is not None:
            self.subject.export(outfile, level, namespace_, name_='subject', pretty_print=pretty_print)
        if self.uid is not None:
            self.uid.export(outfile, level, namespace_, name_='uid', pretty_print=pretty_print)
        if self.accessionNo is not None:
            self.accessionNo.export(outfile, level, namespace_, name_='accessionNo', pretty_print=pretty_print)
        for identifier_ in self.identifier:
            identifier_.export(outfile, level, namespace_, name_='identifier', pretty_print=pretty_print)
        for order_ in self.order:
            order_.export(outfile, level, namespace_, name_='order', pretty_print=pretty_print)
        for modality_ in self.modality:
            modality_.export(outfile, level, namespace_, name_='modality', pretty_print=pretty_print)
        if self.referrer is not None:
            self.referrer.export(outfile, level, namespace_, name_='referrer', pretty_print=pretty_print)
        if self.availability is not None:
            self.availability.export(outfile, level, namespace_, name_='availability', pretty_print=pretty_print)
        if self.url is not None:
            self.url.export(outfile, level, namespace_, name_='url', pretty_print=pretty_print)
        if self.numberOfSeries is not None:
            self.numberOfSeries.export(outfile, level, namespace_, name_='numberOfSeries', pretty_print=pretty_print)
        if self.numberOfInstances is not None:
            self.numberOfInstances.export(outfile, level, namespace_, name_='numberOfInstances', pretty_print=pretty_print)
        if self.clinicalInformation is not None:
            self.clinicalInformation.export(outfile, level, namespace_, name_='clinicalInformation', pretty_print=pretty_print)
        for procedure_ in self.procedure:
            procedure_.export(outfile, level, namespace_, name_='procedure', pretty_print=pretty_print)
        if self.interpreter is not None:
            self.interpreter.export(outfile, level, namespace_, name_='interpreter', pretty_print=pretty_print)
        if self.description is not None:
            self.description.export(outfile, level, namespace_, name_='description', pretty_print=pretty_print)
        for series_ in self.series:
            series_.export(outfile, level, namespace_, name_='series', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='ImagingStudy', mapping_=None):
        element = super(ImagingStudy, self).to_etree(parent_element, name_, mapping_)
        if self.dateTime is not None:
            dateTime_ = self.dateTime
            dateTime_.to_etree(element, name_='dateTime', mapping_=mapping_)
        if self.subject is not None:
            subject_ = self.subject
            subject_.to_etree(element, name_='subject', mapping_=mapping_)
        if self.uid is not None:
            uid_ = self.uid
            uid_.to_etree(element, name_='uid', mapping_=mapping_)
        if self.accessionNo is not None:
            accessionNo_ = self.accessionNo
            accessionNo_.to_etree(element, name_='accessionNo', mapping_=mapping_)
        for identifier_ in self.identifier:
            identifier_.to_etree(element, name_='identifier', mapping_=mapping_)
        for order_ in self.order:
            order_.to_etree(element, name_='order', mapping_=mapping_)
        for modality_ in self.modality:
            modality_.to_etree(element, name_='modality', mapping_=mapping_)
        if self.referrer is not None:
            referrer_ = self.referrer
            referrer_.to_etree(element, name_='referrer', mapping_=mapping_)
        if self.availability is not None:
            availability_ = self.availability
            availability_.to_etree(element, name_='availability', mapping_=mapping_)
        if self.url is not None:
            url_ = self.url
            url_.to_etree(element, name_='url', mapping_=mapping_)
        if self.numberOfSeries is not None:
            numberOfSeries_ = self.numberOfSeries
            numberOfSeries_.to_etree(element, name_='numberOfSeries', mapping_=mapping_)
        if self.numberOfInstances is not None:
            numberOfInstances_ = self.numberOfInstances
            numberOfInstances_.to_etree(element, name_='numberOfInstances', mapping_=mapping_)
        if self.clinicalInformation is not None:
            clinicalInformation_ = self.clinicalInformation
            clinicalInformation_.to_etree(element, name_='clinicalInformation', mapping_=mapping_)
        for procedure_ in self.procedure:
            procedure_.to_etree(element, name_='procedure', mapping_=mapping_)
        if self.interpreter is not None:
            interpreter_ = self.interpreter
            interpreter_.to_etree(element, name_='interpreter', mapping_=mapping_)
        if self.description is not None:
            description_ = self.description
            description_.to_etree(element, name_='description', mapping_=mapping_)
        for series_ in self.series:
            series_.to_etree(element, name_='series', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(ImagingStudy, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'dateTime':
            obj_ = dateTime.factory()
            obj_.build(child_)
            self.dateTime = obj_
            obj_.original_tagname_ = 'dateTime'
        elif nodeName_ == 'subject':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.subject = obj_
            obj_.original_tagname_ = 'subject'
        elif nodeName_ == 'uid':
            obj_ = oid.factory()
            obj_.build(child_)
            self.uid = obj_
            obj_.original_tagname_ = 'uid'
        elif nodeName_ == 'accessionNo':
            obj_ = Identifier.factory()
            obj_.build(child_)
            self.accessionNo = obj_
            obj_.original_tagname_ = 'accessionNo'
        elif nodeName_ == 'identifier':
            obj_ = Identifier.factory()
            obj_.build(child_)
            self.identifier.append(obj_)
            obj_.original_tagname_ = 'identifier'
        elif nodeName_ == 'order':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.order.append(obj_)
            obj_.original_tagname_ = 'order'
        elif nodeName_ == 'modality':
            obj_ = ImagingModality.factory()
            obj_.build(child_)
            self.modality.append(obj_)
            obj_.original_tagname_ = 'modality'
        elif nodeName_ == 'referrer':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.referrer = obj_
            obj_.original_tagname_ = 'referrer'
        elif nodeName_ == 'availability':
            obj_ = InstanceAvailability.factory()
            obj_.build(child_)
            self.availability = obj_
            obj_.original_tagname_ = 'availability'
        elif nodeName_ == 'url':
            obj_ = uri.factory()
            obj_.build(child_)
            self.url = obj_
            obj_.original_tagname_ = 'url'
        elif nodeName_ == 'numberOfSeries':
            obj_ = integer.factory()
            obj_.build(child_)
            self.numberOfSeries = obj_
            obj_.original_tagname_ = 'numberOfSeries'
        elif nodeName_ == 'numberOfInstances':
            obj_ = integer.factory()
            obj_.build(child_)
            self.numberOfInstances = obj_
            obj_.original_tagname_ = 'numberOfInstances'
        elif nodeName_ == 'clinicalInformation':
            obj_ = string.factory()
            obj_.build(child_)
            self.clinicalInformation = obj_
            obj_.original_tagname_ = 'clinicalInformation'
        elif nodeName_ == 'procedure':
            obj_ = Coding.factory()
            obj_.build(child_)
            self.procedure.append(obj_)
            obj_.original_tagname_ = 'procedure'
        elif nodeName_ == 'interpreter':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.interpreter = obj_
            obj_.original_tagname_ = 'interpreter'
        elif nodeName_ == 'description':
            obj_ = string.factory()
            obj_.build(child_)
            self.description = obj_
            obj_.original_tagname_ = 'description'
        elif nodeName_ == 'series':
            obj_ = ImagingStudy_Series.factory()
            obj_.build(child_)
            self.series.append(obj_)
            obj_.original_tagname_ = 'series'
        super(ImagingStudy, self).buildChildren(child_, node, nodeName_, True)
# end class ImagingStudy


class ImagingStudy_Series(BackboneElement):
    """Manifest of a set of images produced in study. The set of images may
    include every image in the study, or it may be an incomplete
    sample, such as a list of key images."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, number=None, modality=None, uid=None, description=None, numberOfInstances=None, availability=None, url=None, bodySite=None, dateTime=None, instance=None):
        self.original_tagname_ = None
        super(ImagingStudy_Series, self).__init__(id, extension, modifierExtension, )
        self.number = number
        self.modality = modality
        self.uid = uid
        self.description = description
        self.numberOfInstances = numberOfInstances
        self.availability = availability
        self.url = url
        self.bodySite = bodySite
        self.dateTime = dateTime
        if instance is None:
            self.instance = []
        else:
            self.instance = instance
    def factory(*args_, **kwargs_):
        if ImagingStudy_Series.subclass:
            return ImagingStudy_Series.subclass(*args_, **kwargs_)
        else:
            return ImagingStudy_Series(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_number(self): return self.number
    def set_number(self, number): self.number = number
    def get_modality(self): return self.modality
    def set_modality(self, modality): self.modality = modality
    def get_uid(self): return self.uid
    def set_uid(self, uid): self.uid = uid
    def get_description(self): return self.description
    def set_description(self, description): self.description = description
    def get_numberOfInstances(self): return self.numberOfInstances
    def set_numberOfInstances(self, numberOfInstances): self.numberOfInstances = numberOfInstances
    def get_availability(self): return self.availability
    def set_availability(self, availability): self.availability = availability
    def get_url(self): return self.url
    def set_url(self, url): self.url = url
    def get_bodySite(self): return self.bodySite
    def set_bodySite(self, bodySite): self.bodySite = bodySite
    def get_dateTime(self): return self.dateTime
    def set_dateTime(self, dateTime): self.dateTime = dateTime
    def get_instance(self): return self.instance
    def set_instance(self, instance): self.instance = instance
    def add_instance(self, value): self.instance.append(value)
    def insert_instance(self, index, value): self.instance[index] = value
    def hasContent_(self):
        if (
            self.number is not None or
            self.modality is not None or
            self.uid is not None or
            self.description is not None or
            self.numberOfInstances is not None or
            self.availability is not None or
            self.url is not None or
            self.bodySite is not None or
            self.dateTime is not None or
            self.instance or
            super(ImagingStudy_Series, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='ImagingStudy.Series', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='ImagingStudy.Series')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='ImagingStudy.Series', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='ImagingStudy.Series'):
        super(ImagingStudy_Series, self).exportAttributes(outfile, level, already_processed, namespace_, name_='ImagingStudy.Series')
    def exportChildren(self, outfile, level, namespace_='', name_='ImagingStudy.Series', fromsubclass_=False, pretty_print=True):
        super(ImagingStudy_Series, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.number is not None:
            self.number.export(outfile, level, namespace_, name_='number', pretty_print=pretty_print)
        if self.modality is not None:
            self.modality.export(outfile, level, namespace_, name_='modality', pretty_print=pretty_print)
        if self.uid is not None:
            self.uid.export(outfile, level, namespace_, name_='uid', pretty_print=pretty_print)
        if self.description is not None:
            self.description.export(outfile, level, namespace_, name_='description', pretty_print=pretty_print)
        if self.numberOfInstances is not None:
            self.numberOfInstances.export(outfile, level, namespace_, name_='numberOfInstances', pretty_print=pretty_print)
        if self.availability is not None:
            self.availability.export(outfile, level, namespace_, name_='availability', pretty_print=pretty_print)
        if self.url is not None:
            self.url.export(outfile, level, namespace_, name_='url', pretty_print=pretty_print)
        if self.bodySite is not None:
            self.bodySite.export(outfile, level, namespace_, name_='bodySite', pretty_print=pretty_print)
        if self.dateTime is not None:
            self.dateTime.export(outfile, level, namespace_, name_='dateTime', pretty_print=pretty_print)
        for instance_ in self.instance:
            instance_.export(outfile, level, namespace_, name_='instance', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='ImagingStudy.Series', mapping_=None):
        element = super(ImagingStudy_Series, self).to_etree(parent_element, name_, mapping_)
        if self.number is not None:
            number_ = self.number
            number_.to_etree(element, name_='number', mapping_=mapping_)
        if self.modality is not None:
            modality_ = self.modality
            modality_.to_etree(element, name_='modality', mapping_=mapping_)
        if self.uid is not None:
            uid_ = self.uid
            uid_.to_etree(element, name_='uid', mapping_=mapping_)
        if self.description is not None:
            description_ = self.description
            description_.to_etree(element, name_='description', mapping_=mapping_)
        if self.numberOfInstances is not None:
            numberOfInstances_ = self.numberOfInstances
            numberOfInstances_.to_etree(element, name_='numberOfInstances', mapping_=mapping_)
        if self.availability is not None:
            availability_ = self.availability
            availability_.to_etree(element, name_='availability', mapping_=mapping_)
        if self.url is not None:
            url_ = self.url
            url_.to_etree(element, name_='url', mapping_=mapping_)
        if self.bodySite is not None:
            bodySite_ = self.bodySite
            bodySite_.to_etree(element, name_='bodySite', mapping_=mapping_)
        if self.dateTime is not None:
            dateTime_ = self.dateTime
            dateTime_.to_etree(element, name_='dateTime', mapping_=mapping_)
        for instance_ in self.instance:
            instance_.to_etree(element, name_='instance', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(ImagingStudy_Series, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'number':
            obj_ = integer.factory()
            obj_.build(child_)
            self.number = obj_
            obj_.original_tagname_ = 'number'
        elif nodeName_ == 'modality':
            obj_ = Modality.factory()
            obj_.build(child_)
            self.modality = obj_
            obj_.original_tagname_ = 'modality'
        elif nodeName_ == 'uid':
            obj_ = oid.factory()
            obj_.build(child_)
            self.uid = obj_
            obj_.original_tagname_ = 'uid'
        elif nodeName_ == 'description':
            obj_ = string.factory()
            obj_.build(child_)
            self.description = obj_
            obj_.original_tagname_ = 'description'
        elif nodeName_ == 'numberOfInstances':
            obj_ = integer.factory()
            obj_.build(child_)
            self.numberOfInstances = obj_
            obj_.original_tagname_ = 'numberOfInstances'
        elif nodeName_ == 'availability':
            obj_ = InstanceAvailability.factory()
            obj_.build(child_)
            self.availability = obj_
            obj_.original_tagname_ = 'availability'
        elif nodeName_ == 'url':
            obj_ = uri.factory()
            obj_.build(child_)
            self.url = obj_
            obj_.original_tagname_ = 'url'
        elif nodeName_ == 'bodySite':
            obj_ = Coding.factory()
            obj_.build(child_)
            self.bodySite = obj_
            obj_.original_tagname_ = 'bodySite'
        elif nodeName_ == 'dateTime':
            obj_ = dateTime.factory()
            obj_.build(child_)
            self.dateTime = obj_
            obj_.original_tagname_ = 'dateTime'
        elif nodeName_ == 'instance':
            obj_ = ImagingStudy_Instance.factory()
            obj_.build(child_)
            self.instance.append(obj_)
            obj_.original_tagname_ = 'instance'
        super(ImagingStudy_Series, self).buildChildren(child_, node, nodeName_, True)
# end class ImagingStudy_Series


class ImagingStudy_Instance(BackboneElement):
    """Manifest of a set of images produced in study. The set of images may
    include every image in the study, or it may be an incomplete
    sample, such as a list of key images."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, number=None, uid=None, sopclass=None, type_=None, title=None, url=None, attachment=None):
        self.original_tagname_ = None
        super(ImagingStudy_Instance, self).__init__(id, extension, modifierExtension, )
        self.number = number
        self.uid = uid
        self.sopclass = sopclass
        self.type_ = type_
        self.title = title
        self.url = url
        self.attachment = attachment
    def factory(*args_, **kwargs_):
        if ImagingStudy_Instance.subclass:
            return ImagingStudy_Instance.subclass(*args_, **kwargs_)
        else:
            return ImagingStudy_Instance(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_number(self): return self.number
    def set_number(self, number): self.number = number
    def get_uid(self): return self.uid
    def set_uid(self, uid): self.uid = uid
    def get_sopclass(self): return self.sopclass
    def set_sopclass(self, sopclass): self.sopclass = sopclass
    def get_type(self): return self.type_
    def set_type(self, type_): self.type_ = type_
    def get_title(self): return self.title
    def set_title(self, title): self.title = title
    def get_url(self): return self.url
    def set_url(self, url): self.url = url
    def get_attachment(self): return self.attachment
    def set_attachment(self, attachment): self.attachment = attachment
    def hasContent_(self):
        if (
            self.number is not None or
            self.uid is not None or
            self.sopclass is not None or
            self.type_ is not None or
            self.title is not None or
            self.url is not None or
            self.attachment is not None or
            super(ImagingStudy_Instance, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='ImagingStudy.Instance', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='ImagingStudy.Instance')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='ImagingStudy.Instance', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='ImagingStudy.Instance'):
        super(ImagingStudy_Instance, self).exportAttributes(outfile, level, already_processed, namespace_, name_='ImagingStudy.Instance')
    def exportChildren(self, outfile, level, namespace_='', name_='ImagingStudy.Instance', fromsubclass_=False, pretty_print=True):
        super(ImagingStudy_Instance, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.number is not None:
            self.number.export(outfile, level, namespace_, name_='number', pretty_print=pretty_print)
        if self.uid is not None:
            self.uid.export(outfile, level, namespace_, name_='uid', pretty_print=pretty_print)
        if self.sopclass is not None:
            self.sopclass.export(outfile, level, namespace_, name_='sopclass', pretty_print=pretty_print)
        if self.type_ is not None:
            self.type_.export(outfile, level, namespace_, name_='type', pretty_print=pretty_print)
        if self.title is not None:
            self.title.export(outfile, level, namespace_, name_='title', pretty_print=pretty_print)
        if self.url is not None:
            self.url.export(outfile, level, namespace_, name_='url', pretty_print=pretty_print)
        if self.attachment is not None:
            self.attachment.export(outfile, level, namespace_, name_='attachment', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='ImagingStudy.Instance', mapping_=None):
        element = super(ImagingStudy_Instance, self).to_etree(parent_element, name_, mapping_)
        if self.number is not None:
            number_ = self.number
            number_.to_etree(element, name_='number', mapping_=mapping_)
        if self.uid is not None:
            uid_ = self.uid
            uid_.to_etree(element, name_='uid', mapping_=mapping_)
        if self.sopclass is not None:
            sopclass_ = self.sopclass
            sopclass_.to_etree(element, name_='sopclass', mapping_=mapping_)
        if self.type_ is not None:
            type__ = self.type_
            type__.to_etree(element, name_='type', mapping_=mapping_)
        if self.title is not None:
            title_ = self.title
            title_.to_etree(element, name_='title', mapping_=mapping_)
        if self.url is not None:
            url_ = self.url
            url_.to_etree(element, name_='url', mapping_=mapping_)
        if self.attachment is not None:
            attachment_ = self.attachment
            attachment_.to_etree(element, name_='attachment', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(ImagingStudy_Instance, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'number':
            obj_ = integer.factory()
            obj_.build(child_)
            self.number = obj_
            obj_.original_tagname_ = 'number'
        elif nodeName_ == 'uid':
            obj_ = oid.factory()
            obj_.build(child_)
            self.uid = obj_
            obj_.original_tagname_ = 'uid'
        elif nodeName_ == 'sopclass':
            obj_ = oid.factory()
            obj_.build(child_)
            self.sopclass = obj_
            obj_.original_tagname_ = 'sopclass'
        elif nodeName_ == 'type':
            obj_ = string.factory()
            obj_.build(child_)
            self.type_ = obj_
            obj_.original_tagname_ = 'type'
        elif nodeName_ == 'title':
            obj_ = string.factory()
            obj_.build(child_)
            self.title = obj_
            obj_.original_tagname_ = 'title'
        elif nodeName_ == 'url':
            obj_ = uri.factory()
            obj_.build(child_)
            self.url = obj_
            obj_.original_tagname_ = 'url'
        elif nodeName_ == 'attachment':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.attachment = obj_
            obj_.original_tagname_ = 'attachment'
        super(ImagingStudy_Instance, self).buildChildren(child_, node, nodeName_, True)
# end class ImagingStudy_Instance


class ImagingModality(Element):
    """Type of acquired image data in the instanceIf the element is
    present, it must have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(ImagingModality, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if ImagingModality.subclass:
            return ImagingModality.subclass(*args_, **kwargs_)
        else:
            return ImagingModality(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(ImagingModality, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='ImagingModality', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='ImagingModality')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='ImagingModality', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='ImagingModality'):
        super(ImagingModality, self).exportAttributes(outfile, level, already_processed, namespace_, name_='ImagingModality')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='ImagingModality', fromsubclass_=False, pretty_print=True):
        super(ImagingModality, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='ImagingModality', mapping_=None):
        element = super(ImagingModality, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(ImagingModality, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(ImagingModality, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class ImagingModality


class Modality(Element):
    """Type of data in the instanceIf the element is present, it must have
    either a @value, an @id, or extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(Modality, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if Modality.subclass:
            return Modality.subclass(*args_, **kwargs_)
        else:
            return Modality(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(Modality, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Modality', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Modality')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Modality', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Modality'):
        super(Modality, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Modality')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='Modality', fromsubclass_=False, pretty_print=True):
        super(Modality, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Modality', mapping_=None):
        element = super(Modality, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(Modality, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(Modality, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class Modality



from StringIO import StringIO
from operator import attrgetter
from datetime import datetime
from .datastore import find_record
from .health_mixin import ExportXMLMixin
import server.fhir as supermod
from server.common import safe_attrgetter

try:
    from flask import url_for
    RUN_FLASK=True
except:
    from .datastore import dumb_url_generate
    RUN_FLASK=False

class Procedure_Map:
    model_mapping={
            'gnuhealth.ambulatory_care_procedure':
                {'subject': 'name.patient',
                    'date': 'name.session_start',
                    'type': 'procedure',
                    'description': 'procedure.description',
                    'name': 'procedure.rec_name',
                    'code': 'procedure.name'},
            'gnuhealth.operation':
                {'subject': 'name.patient',
                    'subject_name': 'name.patient.rec_name',
                    'date': 'name.surgery_date',
                    'type': 'procedure',
                    'description': 'procedure.description',
                    'name': 'procedure.rec_name',
                    'code': 'procedure.name'},
            'gnuhealth.rounding_procedure':
                {'subject': 'name.name.patient',
                    'date': 'name.evaluation_start',
                    'type': 'procedure',
                    'description': 'procedure.description',
                    'name': 'procedure.rec_name',
                    'code': 'procedure.name'}}

    url_prefixes={}
                    #'gnuhealth.rounding_procedure': 'rounds',
                    #'gnuhealth.ambulatory_care_procedure': 'amb',
                    #'gnuhealth.operation': 'surg'}

    resource_search_params= {
            '_id': 'token',
            '_language': None,
            'date': 'date',
            'subject': 'reference',
            'type': 'token',
            }

    # Reference parameter to resource type
    chain_map={
            'subject': 'Patient'}

    # Since these models are pretty similar, all can share same mapping
    #       i.e., no weird or special cases
    #       so we can generate the searches from the model_mapping
    #    But for now, ignore other models = only gnuhealth.operation
    t='gnuhealth.operation'
    search_mapping={
            '_id': ['id'],
            'date': [model_mapping[t]['date']],
            'subject': [model_mapping[t]['subject']],
            'type': [model_mapping[t]['code']],
            'type:text': [model_mapping[t]['name'],
                        model_mapping[t]['description']]}

class health_Procedure(supermod.Procedure, Procedure_Map, ExportXMLMixin):
    def __init__(self, *args, **kwargs):
        rec = kwargs.pop('gnu_record', None)
        super(health_Procedure, self).__init__(*args, **kwargs)
        if rec:
            self.set_gnu_procedure(rec)

    def set_gnu_procedure(self, procedure):
        """Set procedure;

        GNU Health uses 'operation' and 'procedure'
        sometimes interchangeably
        """
        self.procedure = procedure
        self.model_type = self.procedure.__name__

        # Only certain models
        if self.model_type not in self.model_mapping:
            raise ValueError('Not a valid model')

        #self.search_prefix = self.url_prefixes[self.model_type]
        self.map = self.model_mapping[self.model_type]

        self.__import_from_gnu_procedure()
        self.__set_feed_info()

    def __import_from_gnu_procedure(self):
        """Set the data from the model"""
        if self.procedure:
            self.set_subject(safe_attrgetter(self.procedure, self.map['subject']))
            self.set_type(safe_attrgetter(self.procedure, self.map['type']))
            self.set_date(safe_attrgetter(self.procedure, self.map['date']),
                            safe_attrgetter(self.procedure, self.map['date']))
            self.set_identifier(
                    safe_attrgetter(self.procedure, self.map['subject_name']),
                    safe_attrgetter(self.procedure, self.map['type']),
                    safe_attrgetter(self.procedure, self.map['date']))

    def __set_feed_info(self):
        """Set the feed-relevant data"""
        if self.procedure:
            if RUN_FLASK:
                uri = url_for('op_record',
                                log_id=self.procedure.id,
                                _external=True)
            else:
                uri = dumb_url_generate(['Procedure',
                                self.procedure.id])
            self.feed={'id': uri,
                    'published': self.procedure.create_date,
                    'updated': self.procedure.write_date or self.procedure.create_date,
                    'title': attrgetter(self.map['name'])(self.procedure)
                        }

    def set_identifier(self, patient, procedure, date):
        """Extends superclass for convenience

        Keyword arguments:
        patient -- patient name
        procedure -- procedure (Health model)
        date -- date (datetime object)
        """

        if procedure and patient and date:
            if RUN_FLASK:
                value = supermod.string(value=url_for('op_record', log_id=self.procedure.id))
            else:
                value = supermod.string(value=dumb_url_generate(['Procedure',
                                                                self.procedure.id]))
            ident = supermod.Identifier(
                    label = supermod.string(
                        value='{0} performed on {1} on {2}'.format(
                            procedure.rec_name, patient, date.strftime('%Y/%m/%d'))),
                        value=value)
            super(health_Procedure, self).add_identifier(ident)

    def set_subject(self, subject):
        """Extends superclass for convenience

        Keyword arguments:
        subject -- the patient (Health model)
        """

        if subject:
            if RUN_FLASK:
                uri = url_for('pat_record', log_id=subject.id)
            else:
                uri = dumb_url_generate(['Patient', subject.id])
            display = subject.rec_name
            ref=supermod.ResourceReference()
            ref.display = supermod.string(value=display)
            ref.reference = supermod.string(value=uri)
            super(health_Procedure, self).set_subject(ref)

    def set_date(self, start, end):
        """Extends superclass for convenience

        Keyword arguments:
        start -- the start date (datetime object)
        end -- the end date (datetime object)
        """
        #TODO Better specifics

        if start is not None:
            p = supermod.Period()
            p.start = supermod.dateTime(value=start.strftime("%Y/%m/%d"))
            if end is not None:
                p.end = supermod.dateTime(value=end.strftime("%Y/%m/%d"))
            super(health_Procedure, self).set_date(p)

    def set_type(self, type_):
        """Extends superclass for convenience

        Keyword arguments:
        type_ -- procedure (Health model)
        """

        if type_:
            concept = supermod.CodeableConcept()
            des= type_.description
            if des:
                concept.text = supermod.string(value=des)
            concept.coding=[supermod.Coding()]
            name = type_.rec_name
            if name:
                concept.coding[0].display = supermod.string(value=name)
                concept.text=supermod.string(value=name)
            concept.coding[0].code=supermod.code(value=type_.name)
            #ICD-10-PCS
            concept.coding[0].system=supermod.uri(value='urn:oid:2.16.840.1.113883.6.4')
            super(health_Procedure, self).set_type(concept)

supermod.Procedure.subclass=health_Procedure



"""Authenticating HTTP Server

This module builds on BaseHTTPServer and implements basic authentication

"""


import base64
import binascii
import http.server


DEFAULT_AUTH_ERROR_MESSAGE = """
<head>
<title>%(code)d - %(message)s</title>
</head>
<body>
<h1>Authorization Required</h1>
this server could not verify that you
are authorized to access the document
requested.  Either you supplied the wrong
credentials (e.g., bad password), or your
browser doesn't understand how to supply
the credentials required.
</body>"""


def _quote_html(html):
    return html.replace("&", "&amp;").replace("<", "&lt;").replace(">", "&gt;")


class AuthRequestHandler(http.server.BaseHTTPRequestHandler):
    """
    Simple handler that can check for auth headers

    In your subclass you have to define the method get_userinfo(user, password)
    which should return 1 or None depending on whether the password was
    ok or not. None means that the user is not authorized.
    """

    # False means no authentiation
    DO_AUTH = 1

    def parse_request(self):
        if not http.server.BaseHTTPRequestHandler.parse_request(self):
            return False

        if self.DO_AUTH:
            authorization = self.headers.get('Authorization', '')
            if not authorization:
                self.send_autherror(401, b"Authorization Required")
                return False
            scheme, credentials = authorization.split()
            if scheme != 'Basic':
                self.send_error(501)
                return False
            credentials = base64.decodebytes(credentials.encode()).decode()
            user, password = credentials.split(':')
            if not self.get_userinfo(user, password, self.command):
                self.send_autherror(401, b"Authorization Required")
                return False
        return True

    def send_autherror(self, code, message=None):
        """Send and log an auth error reply.

        Arguments are the error code, and a detailed message.
        The detailed message defaults to the short entry matching the
        response code.

        This sends an error response (so it must be called before any
        output has been generated), logs the error, and finally sends
        a piece of HTML explaining the error to the user.

        """
        try:
            short, int = self.responses[code]
        except KeyError:
            short, int = '???', '???'
        if message is None:
            message = short
        explain = int
        self.log_error("code %d, message %s", code, message)

        # using _quote_html to prevent Cross Site Scripting attacks (see bug
        # #1100201)
        content = (self.error_auth_message_format % {'code': code, 'message':
                   _quote_html(message), 'explain': explain})
        self.send_response(code, message)
        self.send_header('Content-Type', self.error_content_type)
        self.send_header('WWW-Authenticate', 'Basic realm="PyWebDAV"')
        self.send_header('Connection', 'close')
        self.end_headers()
        self.wfile.write(content.encode('utf-8'))

    error_auth_message_format = DEFAULT_AUTH_ERROR_MESSAGE

    def get_userinfo(self, user, password, command):
        """Checks if the given user and the given
        password are allowed to access.
        """
        # Always reject
        return None



# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    Copyright (C) 2008-2017 Luis Falcon <lfalcon@gnusolidario.org>
#    Copyright (C) 2013 Sebastián Marro <smarro@gnusolidario.org>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from trytond.model import ModelView, ModelSQL, fields

__all__ = ['PediatricsGrowthChartsWHO']


class PediatricsGrowthChartsWHO(ModelSQL, ModelView):
    'Pediatrics Growth Chart WHO'
    __name__ = 'gnuhealth.pediatrics.growth.charts.who'

    indicator = fields.Selection([
        ('l/h-f-a', 'Length/height for age'),
        ('w-f-a', 'Weight for age'),
        ('bmi-f-a', 'Body mass index for age (BMI for age)'),
        ], 'Indicator', sort=False, required=True)
    measure = fields.Selection([
        ('p', 'percentiles'),
        ('z', 'z-scores'),
        ], 'Measure')
    sex = fields.Selection([
        ('m', 'Male'),
        ('f', 'Female'),
        ], 'Sex')
    month = fields.Integer('Month')
    type = fields.Char('Type')
    value = fields.Float('Value')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#    Copyright (C) 2014-2017 Luis Falcon <lfalcon@gnu.org>
#    Copyright (C) 2014-2017 GNU Solidario <health@gnusolidario.org>

#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

from setuptools import setup
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()



setup(name='fhir',
    version='0.0.4',
    description='HL7 FHIR Library',
    author='Luis Falcon',
    author_email='lfalcon@gnusolidario.org',
    url='http://health.gnu.org',
    download_url='http://ftp.gnu.org/gnu/health/',
    package_dir={'fhir': '.'},
    packages=['fhir'],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Plugins',
        'Intended Audience :: Developers',
        'Intended Audience :: Healthcare Industry',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Natural Language :: English',
        'Natural Language :: Spanish',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Topic :: Scientific/Engineering :: Bio-Informatics',
        'Topic :: Scientific/Engineering :: Medical Science Apps.',
        ],
    keywords='HL7 FHIR GNUHEALTH',
    license='GPL-3',
    requires = ['requests'],
)



# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    Copyright (C) 2008-2017 Luis Falcon <falcon@gnu.org>
#    Copyright (C) 2011-2017 GNU Solidario <health@gnusolidario.org>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from trytond.wizard import Wizard, StateView, Button, StateAction
from trytond.model import ModelView, fields
from trytond.transaction import Transaction
from trytond.pool import Pool

__all__ = ['CheckImmunizationStatusInit','CheckImmunizationStatus']

class CheckImmunizationStatusInit(ModelView):
    'Check Immunization Status Init'
    __name__ = 'gnuhealth.check_immunization_status.init'
    immunization_schedule = \
        fields.Many2One("gnuhealth.immunization_schedule","Schedule",
        required=True)

class CheckImmunizationStatus(Wizard):
    'Check Immunization Status'
    __name__ = 'gnuhealth.check_immunization_status'

    start = StateView('gnuhealth.check_immunization_status.init',
            'health.view_check_immunization_status', [
            Button('Cancel', 'end', 'tryton-cancel'),
            Button('Immunization Status', 'check_immunization_status',
                'tryton-ok', True),
            ])
    check_immunization_status = StateAction('health.report_immunization_status')

    def do_check_immunization_status(self, action):
        return action, self.get_info()
        
    def get_info(self):
    
        return {
            'patient_id': Transaction().context.get('active_id'),
            'immunization_schedule_id': self.start.immunization_schedule.id
            }

    def transition_check_immunization_status(self):
        return 'end'



from collections import namedtuple

class Routing:
    """Simple adapter for mapping the resource route
    to the request handler""" 

    def __init__(self, endpoint):
        self.routes = {}
        self.routes['create'] = {'handler': None, 'uri': ['/{}'.format(endpoint)]}
        self.routes['search'] = {'handler': None, 'uri': ['/{}'.format(endpoint),
                                            '/{}/_search'.format(endpoint)]}
        self.routes['record'] = {'handler': None, 'uri': ['/{}/<int:log_id>'.format(endpoint)]}
        self.routes['validate'] = {'handler': None, 'uri':
                                ['/{}/_validate'.format(endpoint),
                                '/{}/_validate/<int:log_id>'.format(endpoint)]}
        self.routes['version'] = {'handler': None, 'uri':
                                    ['/{}/<int:log_id>/_history'.format(endpoint),
                                    '/{}/<int:log_id>/_history/<string:v_id>'.format(endpoint)]}
        self.endpoint = endpoint

    def __len__(self):
        return len(self.routes)

    def __getitem__(self, type_):
        return self.routes[type_]

    def __setitem__(self, type_, handler):
        if type_ not in self.routes:
            raise KeyError('Invalid key')
        self.routes[type_]['handler'] = handler

    def __repr__(self):
        return 'Route %s' % self.endpoint

    @property
    def routing_table(self):
        return self.routes

__all__ = ['Routing']



from .base_classes import *
from .support_functions import *


class Composition(Resource):
    """A set of healthcare-related information that is assembled together
    into a single logical document that provides a single coherent
    statement of meaning, establishes its own context and that has
    clinical attestation with regard to who is making the
    statement.If the element is present, it must have either a
    @value, an @id, or extensions"""
    subclass = None
    superclass = Resource
    def __init__(self, id=None, extension=None, modifierExtension=None, language=None, text=None, contained=None, identifier=None, date=None, type_=None, class_=None, title=None, status=None, confidentiality=None, subject=None, author=None, attester=None, custodian=None, event=None, encounter=None, section=None):
        self.original_tagname_ = None
        super(Composition, self).__init__(id, extension, modifierExtension, language, text, contained, )
        self.identifier = identifier
        self.date = date
        self.type_ = type_
        self.class_ = class_
        self.title = title
        self.status = status
        self.confidentiality = confidentiality
        self.subject = subject
        if author is None:
            self.author = []
        else:
            self.author = author
        if attester is None:
            self.attester = []
        else:
            self.attester = attester
        self.custodian = custodian
        self.event = event
        self.encounter = encounter
        if section is None:
            self.section = []
        else:
            self.section = section
    def factory(*args_, **kwargs_):
        if Composition.subclass:
            return Composition.subclass(*args_, **kwargs_)
        else:
            return Composition(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_identifier(self): return self.identifier
    def set_identifier(self, identifier): self.identifier = identifier
    def get_date(self): return self.date
    def set_date(self, date): self.date = date
    def get_type(self): return self.type_
    def set_type(self, type_): self.type_ = type_
    def get_class(self): return self.class_
    def set_class(self, class_): self.class_ = class_
    def get_title(self): return self.title
    def set_title(self, title): self.title = title
    def get_status(self): return self.status
    def set_status(self, status): self.status = status
    def get_confidentiality(self): return self.confidentiality
    def set_confidentiality(self, confidentiality): self.confidentiality = confidentiality
    def get_subject(self): return self.subject
    def set_subject(self, subject): self.subject = subject
    def get_author(self): return self.author
    def set_author(self, author): self.author = author
    def add_author(self, value): self.author.append(value)
    def insert_author(self, index, value): self.author[index] = value
    def get_attester(self): return self.attester
    def set_attester(self, attester): self.attester = attester
    def add_attester(self, value): self.attester.append(value)
    def insert_attester(self, index, value): self.attester[index] = value
    def get_custodian(self): return self.custodian
    def set_custodian(self, custodian): self.custodian = custodian
    def get_event(self): return self.event
    def set_event(self, event): self.event = event
    def get_encounter(self): return self.encounter
    def set_encounter(self, encounter): self.encounter = encounter
    def get_section(self): return self.section
    def set_section(self, section): self.section = section
    def add_section(self, value): self.section.append(value)
    def insert_section(self, index, value): self.section[index] = value
    def hasContent_(self):
        if (
            self.identifier is not None or
            self.date is not None or
            self.type_ is not None or
            self.class_ is not None or
            self.title is not None or
            self.status is not None or
            self.confidentiality is not None or
            self.subject is not None or
            self.author or
            self.attester or
            self.custodian is not None or
            self.event is not None or
            self.encounter is not None or
            self.section or
            super(Composition, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Composition', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Composition')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Composition', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Composition'):
        super(Composition, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Composition')
    def exportChildren(self, outfile, level, namespace_='', name_='Composition', fromsubclass_=False, pretty_print=True):
        super(Composition, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.identifier is not None:
            self.identifier.export(outfile, level, namespace_, name_='identifier', pretty_print=pretty_print)
        if self.date is not None:
            self.date.export(outfile, level, namespace_, name_='date', pretty_print=pretty_print)
        if self.type_ is not None:
            self.type_.export(outfile, level, namespace_, name_='type', pretty_print=pretty_print)
        if self.class_ is not None:
            self.class_.export(outfile, level, namespace_, name_='class', pretty_print=pretty_print)
        if self.title is not None:
            self.title.export(outfile, level, namespace_, name_='title', pretty_print=pretty_print)
        if self.status is not None:
            self.status.export(outfile, level, namespace_, name_='status', pretty_print=pretty_print)
        if self.confidentiality is not None:
            self.confidentiality.export(outfile, level, namespace_, name_='confidentiality', pretty_print=pretty_print)
        if self.subject is not None:
            self.subject.export(outfile, level, namespace_, name_='subject', pretty_print=pretty_print)
        for author_ in self.author:
            author_.export(outfile, level, namespace_, name_='author', pretty_print=pretty_print)
        for attester_ in self.attester:
            attester_.export(outfile, level, namespace_, name_='attester', pretty_print=pretty_print)
        if self.custodian is not None:
            self.custodian.export(outfile, level, namespace_, name_='custodian', pretty_print=pretty_print)
        if self.event is not None:
            self.event.export(outfile, level, namespace_, name_='event', pretty_print=pretty_print)
        if self.encounter is not None:
            self.encounter.export(outfile, level, namespace_, name_='encounter', pretty_print=pretty_print)
        for section_ in self.section:
            section_.export(outfile, level, namespace_, name_='section', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Composition', mapping_=None):
        element = super(Composition, self).to_etree(parent_element, name_, mapping_)
        if self.identifier is not None:
            identifier_ = self.identifier
            identifier_.to_etree(element, name_='identifier', mapping_=mapping_)
        if self.date is not None:
            date_ = self.date
            date_.to_etree(element, name_='date', mapping_=mapping_)
        if self.type_ is not None:
            type__ = self.type_
            type__.to_etree(element, name_='type', mapping_=mapping_)
        if self.class_ is not None:
            class__ = self.class_
            class__.to_etree(element, name_='class', mapping_=mapping_)
        if self.title is not None:
            title_ = self.title
            title_.to_etree(element, name_='title', mapping_=mapping_)
        if self.status is not None:
            status_ = self.status
            status_.to_etree(element, name_='status', mapping_=mapping_)
        if self.confidentiality is not None:
            confidentiality_ = self.confidentiality
            confidentiality_.to_etree(element, name_='confidentiality', mapping_=mapping_)
        if self.subject is not None:
            subject_ = self.subject
            subject_.to_etree(element, name_='subject', mapping_=mapping_)
        for author_ in self.author:
            author_.to_etree(element, name_='author', mapping_=mapping_)
        for attester_ in self.attester:
            attester_.to_etree(element, name_='attester', mapping_=mapping_)
        if self.custodian is not None:
            custodian_ = self.custodian
            custodian_.to_etree(element, name_='custodian', mapping_=mapping_)
        if self.event is not None:
            event_ = self.event
            event_.to_etree(element, name_='event', mapping_=mapping_)
        if self.encounter is not None:
            encounter_ = self.encounter
            encounter_.to_etree(element, name_='encounter', mapping_=mapping_)
        for section_ in self.section:
            section_.to_etree(element, name_='section', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(Composition, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'identifier':
            obj_ = Identifier.factory()
            obj_.build(child_)
            self.identifier = obj_
            obj_.original_tagname_ = 'identifier'
        elif nodeName_ == 'date':
            obj_ = dateTime.factory()
            obj_.build(child_)
            self.date = obj_
            obj_.original_tagname_ = 'date'
        elif nodeName_ == 'type':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.type_ = obj_
            obj_.original_tagname_ = 'type'
        elif nodeName_ == 'class':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.class_ = obj_
            obj_.original_tagname_ = 'class'
        elif nodeName_ == 'title':
            obj_ = string.factory()
            obj_.build(child_)
            self.title = obj_
            obj_.original_tagname_ = 'title'
        elif nodeName_ == 'status':
            obj_ = CompositionStatus.factory()
            obj_.build(child_)
            self.status = obj_
            obj_.original_tagname_ = 'status'
        elif nodeName_ == 'confidentiality':
            obj_ = Coding.factory()
            obj_.build(child_)
            self.confidentiality = obj_
            obj_.original_tagname_ = 'confidentiality'
        elif nodeName_ == 'subject':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.subject = obj_
            obj_.original_tagname_ = 'subject'
        elif nodeName_ == 'author':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.author.append(obj_)
            obj_.original_tagname_ = 'author'
        elif nodeName_ == 'attester':
            obj_ = Composition_Attester.factory()
            obj_.build(child_)
            self.attester.append(obj_)
            obj_.original_tagname_ = 'attester'
        elif nodeName_ == 'custodian':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.custodian = obj_
            obj_.original_tagname_ = 'custodian'
        elif nodeName_ == 'event':
            obj_ = Composition_Event.factory()
            obj_.build(child_)
            self.event = obj_
            obj_.original_tagname_ = 'event'
        elif nodeName_ == 'encounter':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.encounter = obj_
            obj_.original_tagname_ = 'encounter'
        elif nodeName_ == 'section':
            obj_ = Composition_Section.factory()
            obj_.build(child_)
            self.section.append(obj_)
            obj_.original_tagname_ = 'section'
        super(Composition, self).buildChildren(child_, node, nodeName_, True)
# end class Composition


class Composition_Attester(BackboneElement):
    """A set of healthcare-related information that is assembled together
    into a single logical document that provides a single coherent
    statement of meaning, establishes its own context and that has
    clinical attestation with regard to who is making the statement."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, mode=None, time=None, party=None):
        self.original_tagname_ = None
        super(Composition_Attester, self).__init__(id, extension, modifierExtension, )
        if mode is None:
            self.mode = []
        else:
            self.mode = mode
        self.time = time
        self.party = party
    def factory(*args_, **kwargs_):
        if Composition_Attester.subclass:
            return Composition_Attester.subclass(*args_, **kwargs_)
        else:
            return Composition_Attester(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_mode(self): return self.mode
    def set_mode(self, mode): self.mode = mode
    def add_mode(self, value): self.mode.append(value)
    def insert_mode(self, index, value): self.mode[index] = value
    def get_time(self): return self.time
    def set_time(self, time): self.time = time
    def get_party(self): return self.party
    def set_party(self, party): self.party = party
    def hasContent_(self):
        if (
            self.mode or
            self.time is not None or
            self.party is not None or
            super(Composition_Attester, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Composition.Attester', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Composition.Attester')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Composition.Attester', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Composition.Attester'):
        super(Composition_Attester, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Composition.Attester')
    def exportChildren(self, outfile, level, namespace_='', name_='Composition.Attester', fromsubclass_=False, pretty_print=True):
        super(Composition_Attester, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for mode_ in self.mode:
            mode_.export(outfile, level, namespace_, name_='mode', pretty_print=pretty_print)
        if self.time is not None:
            self.time.export(outfile, level, namespace_, name_='time', pretty_print=pretty_print)
        if self.party is not None:
            self.party.export(outfile, level, namespace_, name_='party', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Composition.Attester', mapping_=None):
        element = super(Composition_Attester, self).to_etree(parent_element, name_, mapping_)
        for mode_ in self.mode:
            mode_.to_etree(element, name_='mode', mapping_=mapping_)
        if self.time is not None:
            time_ = self.time
            time_.to_etree(element, name_='time', mapping_=mapping_)
        if self.party is not None:
            party_ = self.party
            party_.to_etree(element, name_='party', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(Composition_Attester, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'mode':
            obj_ = CompositionAttestationMode.factory()
            obj_.build(child_)
            self.mode.append(obj_)
            obj_.original_tagname_ = 'mode'
        elif nodeName_ == 'time':
            obj_ = dateTime.factory()
            obj_.build(child_)
            self.time = obj_
            obj_.original_tagname_ = 'time'
        elif nodeName_ == 'party':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.party = obj_
            obj_.original_tagname_ = 'party'
        super(Composition_Attester, self).buildChildren(child_, node, nodeName_, True)
# end class Composition_Attester


class Composition_Event(BackboneElement):
    """A set of healthcare-related information that is assembled together
    into a single logical document that provides a single coherent
    statement of meaning, establishes its own context and that has
    clinical attestation with regard to who is making the statement."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, code=None, period=None, detail=None):
        self.original_tagname_ = None
        super(Composition_Event, self).__init__(id, extension, modifierExtension, )
        if code is None:
            self.code = []
        else:
            self.code = code
        self.period = period
        if detail is None:
            self.detail = []
        else:
            self.detail = detail
    def factory(*args_, **kwargs_):
        if Composition_Event.subclass:
            return Composition_Event.subclass(*args_, **kwargs_)
        else:
            return Composition_Event(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_code(self): return self.code
    def set_code(self, code): self.code = code
    def add_code(self, value): self.code.append(value)
    def insert_code(self, index, value): self.code[index] = value
    def get_period(self): return self.period
    def set_period(self, period): self.period = period
    def get_detail(self): return self.detail
    def set_detail(self, detail): self.detail = detail
    def add_detail(self, value): self.detail.append(value)
    def insert_detail(self, index, value): self.detail[index] = value
    def hasContent_(self):
        if (
            self.code or
            self.period is not None or
            self.detail or
            super(Composition_Event, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Composition.Event', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Composition.Event')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Composition.Event', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Composition.Event'):
        super(Composition_Event, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Composition.Event')
    def exportChildren(self, outfile, level, namespace_='', name_='Composition.Event', fromsubclass_=False, pretty_print=True):
        super(Composition_Event, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for code_ in self.code:
            code_.export(outfile, level, namespace_, name_='code', pretty_print=pretty_print)
        if self.period is not None:
            self.period.export(outfile, level, namespace_, name_='period', pretty_print=pretty_print)
        for detail_ in self.detail:
            detail_.export(outfile, level, namespace_, name_='detail', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Composition.Event', mapping_=None):
        element = super(Composition_Event, self).to_etree(parent_element, name_, mapping_)
        for code_ in self.code:
            code_.to_etree(element, name_='code', mapping_=mapping_)
        if self.period is not None:
            period_ = self.period
            period_.to_etree(element, name_='period', mapping_=mapping_)
        for detail_ in self.detail:
            detail_.to_etree(element, name_='detail', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(Composition_Event, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'code':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.code.append(obj_)
            obj_.original_tagname_ = 'code'
        elif nodeName_ == 'period':
            obj_ = Period.factory()
            obj_.build(child_)
            self.period = obj_
            obj_.original_tagname_ = 'period'
        elif nodeName_ == 'detail':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.detail.append(obj_)
            obj_.original_tagname_ = 'detail'
        super(Composition_Event, self).buildChildren(child_, node, nodeName_, True)
# end class Composition_Event


class Composition_Section(BackboneElement):
    """A set of healthcare-related information that is assembled together
    into a single logical document that provides a single coherent
    statement of meaning, establishes its own context and that has
    clinical attestation with regard to who is making the statement."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, title=None, code=None, subject=None, content=None, section=None):
        self.original_tagname_ = None
        super(Composition_Section, self).__init__(id, extension, modifierExtension, )
        self.title = title
        self.code = code
        self.subject = subject
        self.content = content
        if section is None:
            self.section = []
        else:
            self.section = section
    def factory(*args_, **kwargs_):
        if Composition_Section.subclass:
            return Composition_Section.subclass(*args_, **kwargs_)
        else:
            return Composition_Section(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_title(self): return self.title
    def set_title(self, title): self.title = title
    def get_code(self): return self.code
    def set_code(self, code): self.code = code
    def get_subject(self): return self.subject
    def set_subject(self, subject): self.subject = subject
    def get_content(self): return self.content
    def set_content(self, content): self.content = content
    def get_section(self): return self.section
    def set_section(self, section): self.section = section
    def add_section(self, value): self.section.append(value)
    def insert_section(self, index, value): self.section[index] = value
    def hasContent_(self):
        if (
            self.title is not None or
            self.code is not None or
            self.subject is not None or
            self.content is not None or
            self.section or
            super(Composition_Section, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Composition.Section', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Composition.Section')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Composition.Section', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Composition.Section'):
        super(Composition_Section, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Composition.Section')
    def exportChildren(self, outfile, level, namespace_='', name_='Composition.Section', fromsubclass_=False, pretty_print=True):
        super(Composition_Section, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.title is not None:
            self.title.export(outfile, level, namespace_, name_='title', pretty_print=pretty_print)
        if self.code is not None:
            self.code.export(outfile, level, namespace_, name_='code', pretty_print=pretty_print)
        if self.subject is not None:
            self.subject.export(outfile, level, namespace_, name_='subject', pretty_print=pretty_print)
        if self.content is not None:
            self.content.export(outfile, level, namespace_, name_='content', pretty_print=pretty_print)
        for section_ in self.section:
            section_.export(outfile, level, namespace_, name_='section', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Composition.Section', mapping_=None):
        element = super(Composition_Section, self).to_etree(parent_element, name_, mapping_)
        if self.title is not None:
            title_ = self.title
            title_.to_etree(element, name_='title', mapping_=mapping_)
        if self.code is not None:
            code_ = self.code
            code_.to_etree(element, name_='code', mapping_=mapping_)
        if self.subject is not None:
            subject_ = self.subject
            subject_.to_etree(element, name_='subject', mapping_=mapping_)
        if self.content is not None:
            content_ = self.content
            content_.to_etree(element, name_='content', mapping_=mapping_)
        for section_ in self.section:
            section_.to_etree(element, name_='section', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(Composition_Section, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'title':
            obj_ = string.factory()
            obj_.build(child_)
            self.title = obj_
            obj_.original_tagname_ = 'title'
        elif nodeName_ == 'code':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.code = obj_
            obj_.original_tagname_ = 'code'
        elif nodeName_ == 'subject':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.subject = obj_
            obj_.original_tagname_ = 'subject'
        elif nodeName_ == 'content':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.content = obj_
            obj_.original_tagname_ = 'content'
        elif nodeName_ == 'section':
            obj_ = Composition_Section.factory()
            obj_.build(child_)
            self.section.append(obj_)
            obj_.original_tagname_ = 'section'
        super(Composition_Section, self).buildChildren(child_, node, nodeName_, True)
# end class Composition_Section


class CompositionStatus(Element):
    """The workflow/clinical status of the compositionIf the element is
    present, it must have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(CompositionStatus, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if CompositionStatus.subclass:
            return CompositionStatus.subclass(*args_, **kwargs_)
        else:
            return CompositionStatus(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(CompositionStatus, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='CompositionStatus', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='CompositionStatus')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='CompositionStatus', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='CompositionStatus'):
        super(CompositionStatus, self).exportAttributes(outfile, level, already_processed, namespace_, name_='CompositionStatus')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='CompositionStatus', fromsubclass_=False, pretty_print=True):
        super(CompositionStatus, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='CompositionStatus', mapping_=None):
        element = super(CompositionStatus, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(CompositionStatus, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(CompositionStatus, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class CompositionStatus


class CompositionAttestationMode(Element):
    """The way in which a person authenticated a compositionIf the element
    is present, it must have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(CompositionAttestationMode, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if CompositionAttestationMode.subclass:
            return CompositionAttestationMode.subclass(*args_, **kwargs_)
        else:
            return CompositionAttestationMode(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(CompositionAttestationMode, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='CompositionAttestationMode', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='CompositionAttestationMode')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='CompositionAttestationMode', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='CompositionAttestationMode'):
        super(CompositionAttestationMode, self).exportAttributes(outfile, level, already_processed, namespace_, name_='CompositionAttestationMode')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='CompositionAttestationMode', fromsubclass_=False, pretty_print=True):
        super(CompositionAttestationMode, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='CompositionAttestationMode', mapping_=None):
        element = super(CompositionAttestationMode, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(CompositionAttestationMode, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(CompositionAttestationMode, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class CompositionAttestationMode
